package j2eetut.enroller;

/*
 *  Copyright 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 *  This software is the proprietary information of Sun Microsystems, Inc.
 *  Use is subject to license terms.
 *
 */
import java.rmi.RemoteException;
import javax.ejb.*;
import java.sql.*;
import javax.sql.*;
import java.util.*;
import javax.naming.*;

public class EnrollerBean implements SessionBean {

    private Connection con;
    private String dbName = "java:comp/env/jdbc/CollegeDB";
    private SessionContext context;


    public void enroll(String studentId, String courseId) {

        try {
            insertEntry(studentId, courseId);
        } catch (Exception ex) {
            throw new EJBException("enroll: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void unEnroll(String studentId, String courseId) {

        try {
            deleteEntry(studentId, courseId);
        } catch (Exception ex) {
            throw new EJBException("unEnroll: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void deleteStudent(String studentId) {

        try {
            deleteStudentEntries(studentId);
        } catch (Exception ex) {
            throw new EJBException("deleteStudent: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void deleteCourse(String courseId) {

        try {
            deleteCourseEntries(courseId);
        } catch (Exception ex) {
            throw new EJBException("deleteCourse: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public ArrayList getStudentIds(String courseId) {

        try {
            return selectStudent(courseId);
        } catch (Exception ex) {
            throw new EJBException("getStudentIds: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public ArrayList getCourseIds(String studentId) {

        try {
            return selectCourse(studentId);
        } catch (Exception ex) {
            throw new EJBException("getCourseIds: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void ejbCreate() {
        /*
        try {
            makeConnection();
        } catch (Exception ex) {
            throw new EJBException("Unable to connect to database. " +
                    ex.getMessage());
        }
        */
    }


    public void ejbRemove() {

        try {
            con.close();
        } catch (SQLException ex) {
            throw new EJBException("ejbRemove: " + ex.getMessage());
        }
    }


    public void ejbActivate() {
        /*
        try {
            con = getConnection();
        } catch (Exception ex) {
            throw new EJBException("ejbActivate Exception: " + ex.getMessage());
        }
        */
    }


    public void ejbPassivate() {
        /*
        try {
            con.close();
        } catch (SQLException ex) {
            throw new EJBException("ejbPassivate Exception: " + ex.getMessage());
        }
        */
    }


    public void setSessionContext(SessionContext context) {
        this.context = context;
    }


    public EnrollerBean() { }


    /**
     *  Database Routines ************************
     *
     *@exception  NamingException  Description of the Exception
     *@exception  SQLException     Description of the Exception
     */
     /*
    private void makeConnection() throws NamingException, SQLException {

        InitialContext ic = new InitialContext();
        DataSource ds = (DataSource) ic.lookup(dbName);
        con = ds.getConnection();
    }
    */
    /**
     *  Changed to avoid transaction deadlock
     *
     *@return                   The connection value
     *@exception  SQLException  Description of the Exception
     */
    private Connection getConnection() throws SQLException {
        try {
            InitialContext ic = new InitialContext();
            DataSource ds = (DataSource) ic.lookup(dbName);
            return ds.getConnection();
        } catch (Exception e) {
            if (e instanceof SQLException) {
                throw (SQLException) e;
            }
            throw new SQLException("Unable to connect to database: " + e);
        }
    }


    /**
     *  Call this in finally clause of all methods
     */
    private void freeConnection() {
        try {
            if (con != null) {
                con.close();
            }
        } catch (Exception e) {}
    }

    private void insertEntry(String studentId, String courseId)
             throws SQLException {
                 
        con = getConnection();
        String insertStatement =
                "insert into enrollment values ( ? , ? )";
        PreparedStatement prepStmt =
                con.prepareStatement(insertStatement);

        prepStmt.setString(1, studentId);
        prepStmt.setString(2, courseId);

        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private void deleteEntry(String studentId, String courseId)
             throws SQLException {

        con = getConnection();
        String deleteStatement =
                "delete from enrollment " +
                "where studentid = ? and courseid = ?";
        PreparedStatement prepStmt =
                con.prepareStatement(deleteStatement);

        prepStmt.setString(1, studentId);
        prepStmt.setString(2, courseId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private void deleteStudentEntries(String studentId)
             throws SQLException {

        con = getConnection();
        String deleteStatement =
                "delete from enrollment " +
                "where studentid = ?";
        PreparedStatement prepStmt =
                con.prepareStatement(deleteStatement);

        prepStmt.setString(1, studentId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private void deleteCourseEntries(String courseId)
             throws SQLException {

        con = getConnection();
        String deleteStatement =
                "delete from enrollment " +
                "where courseid = ?";
        PreparedStatement prepStmt =
                con.prepareStatement(deleteStatement);

        prepStmt.setString(1, courseId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private ArrayList selectStudent(String courseId)
             throws SQLException {

        con = getConnection();
        String selectStatement =
                "select studentid " +
                "from enrollment where courseid = ? ";
        PreparedStatement prepStmt =
                con.prepareStatement(selectStatement);

        prepStmt.setString(1, courseId);
        ResultSet rs = prepStmt.executeQuery();
        ArrayList a = new ArrayList();

        while (rs.next()) {
            String id = rs.getString(1);
            a.add(id);
        }

        prepStmt.close();
        return a;
    }


    private ArrayList selectCourse(String studentId)
             throws SQLException {

        con = getConnection();
        String selectStatement =
                "select courseid " +
                "from enrollment where studentid = ? ";
        PreparedStatement prepStmt =
                con.prepareStatement(selectStatement);

        prepStmt.setString(1, studentId);
        ResultSet rs = prepStmt.executeQuery();
        ArrayList a = new ArrayList();

        while (rs.next()) {
            String id = rs.getString(1);
            a.add(id);
        }

        prepStmt.close();
        return a;
    }

}

