package j2eetut.storagebin;

/*
 *  Copyright 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 *  This software is the proprietary information of Sun Microsystems, Inc.
 *  Use is subject to license terms.
 *
 */
import java.sql.*;
import javax.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;

public class WidgetBean implements EntityBean {

    private String widgetId;
    private String description;
    private double price;
    private EntityContext context;
    private Connection con;
    private String logicalDBName = "java:comp/env/jdbc/WidgetDB";


    public String getDescription() {

        return description;
    }


    public double getPrice() {

        return price;
    }


    public String ejbCreate(String widgetId, String description,
            double price)
             throws CreateException {

        try {
            insertRow(widgetId, description, price);
        } catch (Exception ex) {
            throw new EJBException("ejbCreate: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }

        this.widgetId = widgetId;
        this.description = description;
        this.price = price;

        return widgetId;
    }


    public String ejbFindByPrimaryKey(String primaryKey)
             throws FinderException {

        boolean result;

        try {
            result = selectByPrimaryKey(primaryKey);
        } catch (Exception ex) {
            throw new EJBException("ejbFindByPrimaryKey: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }

        if (result) {
            return primaryKey;
        } else {
            throw new ObjectNotFoundException
                    ("Row for id " + primaryKey + " not found.");
        }
    }


    public void ejbRemove() {

        try {
            deleteRow(widgetId);
        } catch (Exception ex) {
            throw new EJBException("ejbRemove: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void setEntityContext(EntityContext context) {

        this.context = context;
        /*
         *  try {
         *  makeConnection();
         *  } catch (Exception ex) {
         *  throw new EJBException("Unable to connect to database. " +
         *  ex.getMessage());
         *  }
         */
    }


    public void unsetEntityContext() {
        /*
         *  try {
         *  con.close();
         *  } catch (SQLException ex) {
         *  throw new EJBException("unsetEntityContext: " + ex.getMessage());
         *  }
         */
    }


    public void ejbActivate() {

        widgetId = (String) context.getPrimaryKey();
    }


    public void ejbPassivate() {

        widgetId = null;
    }


    public void ejbLoad() {

        try {
            loadRow();
        } catch (Exception ex) {
            throw new EJBException("ejbLoad: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void ejbStore() {

        try {
            storeRow();
        } catch (Exception ex) {
            throw new EJBException("ejbLoad: " +
                    ex.getMessage());
        } finally {
            freeConnection();
        }
    }


    public void ejbPostCreate(String widgetId, String description,
            double price) { }


    /**
     *  Database Routines ************************
     *
     *@return                   The connection value
     *@exception  SQLException  Description of the Exception
     */
    /*
     *  private void makeConnection() throws NamingException, SQLException {
     *  InitialContext ic = new InitialContext();
     *  DataSource ds = (DataSource) ic.lookup(dbName);
     *  con =  ds.getConnection();
     *  }
     */
    /**
     *  Changed to avoid transaction deadlock
     *
     *@return                   The connection value
     *@exception  SQLException  Description of the Exception
     */
    private Connection getConnection() throws SQLException {
        try {
            InitialContext ic = new InitialContext();
            DataSource ds = (DataSource) ic.lookup(logicalDBName);
            return ds.getConnection();
        } catch (Exception e) {
            if (e instanceof SQLException) {
                throw (SQLException) e;
            }
            throw new SQLException("Unable to connect to database: " + e);
        }
    }


    /**
     *  Call this in finally clause of all methods
     */
    private void freeConnection() {
        try {
            if (con != null) {
                con.close();
            }
        } catch (Exception e) {}
    }



    private void insertRow(String widgetId, String description,
            double price) throws SQLException {

        con = getConnection();
        // added to all methods
        String insertStatement =
                "insert into widget values ( ? , ? , ? )";
        PreparedStatement prepStmt =
                con.prepareStatement(insertStatement);

        prepStmt.setString(1, widgetId);
        prepStmt.setString(2, description);
        prepStmt.setDouble(3, price);

        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private void deleteRow(String widgetId) throws SQLException {

        con = getConnection();
        // added to all methods
        String deleteStatement =
                "delete from widget where widgetid = ? ";
        PreparedStatement prepStmt =
                con.prepareStatement(deleteStatement);

        prepStmt.setString(1, widgetId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }


    private boolean selectByPrimaryKey(String primaryKey)
             throws SQLException {

        con = getConnection();
        // added to all methods
        String selectStatement =
                "select widgetid " +
                "from widget where widgetid = ? ";
        PreparedStatement prepStmt =
                con.prepareStatement(selectStatement);
        prepStmt.setString(1, primaryKey);

        ResultSet rs = prepStmt.executeQuery();
        boolean result = rs.next();
        prepStmt.close();
        return result;
    }


    private void loadRow() throws SQLException {

        con = getConnection();
        // added to all methods
        String selectStatement =
                "select description, price " +
                "from widget where widgetid = ? ";
        PreparedStatement prepStmt =
                con.prepareStatement(selectStatement);

        prepStmt.setString(1, this.widgetId);

        ResultSet rs = prepStmt.executeQuery();

        if (rs.next()) {
            this.description = rs.getString(1);
            this.price = rs.getDouble(2);
            prepStmt.close();
        } else {
            prepStmt.close();
            throw new NoSuchEntityException("Row for widgetId " + widgetId +
                    " not found in database.");
        }
    }


    private void storeRow() throws SQLException {

        con = getConnection();
        // added to all methods
        String updateStatement =
                "update widget set description =  ? , " +
                "price = ? " +
                "where widgetid = ?";
        PreparedStatement prepStmt =
                con.prepareStatement(updateStatement);

        prepStmt.setString(1, description);
        prepStmt.setDouble(2, price);
        prepStmt.setString(3, widgetId);
        int rowCount = prepStmt.executeUpdate();
        prepStmt.close();

        if (rowCount == 0) {
            throw new EJBException("Storing row for widgetId " +
                    widgetId + " failed.");
        }
    }

}

