package loja.integration;

import loja.*;

import java.sql.*;
import java.util.*;

/**
create table compras (
	id integer constraint pk_compras primary key,
	cliente varchar(8),
	status varchar(16),
	constraint fk_cliente foreign key (cliente) references clientes(id)
);
*/


///////// TO FIX - LOAD - ///////////////
public class JdbcCompraDAO implements CompraDAO {
	
	/** SQL queries */
	public static final String SQL_NUMERO  = "select max(id) from compras";
	public static final String SQL_CREATE  = "insert into compras(id, status) values (?, ?)";
	public static final String SQL_REMOVE  = "delete from compras where id = ?";
	public static final String SQL_LOAD    = "select id, status from compras where id = ?";
	public static final String SQL_STORE   = "update compras set status = ? where id = ?";
	public static final String SQL_PK      = "select id from compras where id = ?";
	public static final String SQL_ALL     = "select id from compras";
	public static final String SQL_STATUS  = "select id from compras where status = ?";
	
	private static CompraDAO instance = null;
	private ServiceLocator locator;
	
	private JdbcCompraDAO() throws LojaException {
		locator = ServiceLocator.getInstance();
	}
	
	private Connection getConnection() throws LojaException {
		Connection con = locator.findJdbcConnection("jdbc/LojaDB");
		return con;
	}
	
	/**
	 * Retorna a instancia desta classe.
	 */
	public static CompraDAO getInstance()  throws LojaException {
		if (instance == null) {
			instance = new JdbcCompraDAO();
		}
		return instance;
	}
	
	/**
	 * Cria um novo numero univoco para pedido de compra
	 */
	public String criarNumero() throws LojaException {
		String sql = SQL_NUMERO;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String numero = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				int ultimo = rs.getInt(1);
				numero = "" + (ultimo + 1);
			} else {
				numero = "1";
			}
			if (numero == null) {
				throw new SQLException("Incapaz de criar novo numero!");
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return numero;
	}
	
	/**
	 * Usa dados de value object para criar novo registro no banco
	 */
	public String create(CompraVO compra) throws LojaException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String numero = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			numero = compra.getNumero();
			stmt.setInt(1, Integer.parseInt(numero));
			stmt.setString(2, compra.getStatus().toString());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return numero;
	}
	
	/**
	 * Remove item, identificado por sua chave primaria, do banco
	 */
	public void remove(String numero) throws LojaException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	/**
	 * Carrega dados do banco para um value object
	 */
	public CompraVO load(String numero) throws LojaException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		CompraVO compra = new CompraVO();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				compra.setNumero(""+rs.getInt(1));
				compra.setStatus(rs.getString(2));
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return compra;
	}
	
	/**
	 * Armazena dados de value object no banco.
	 */
	public void store(CompraVO compra) throws LojaException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, compra.getStatus().toString());
			stmt.setInt(2, Integer.parseInt(compra.getNumero()));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	/**
	 * Retorna true se chave primaria existe
	 */
	public boolean exists(String numero) throws LojaException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		int numeroLido = -1;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(numero));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				numeroLido = rs.getInt(1);
			}
			if (numeroLido == -1) {
				throw new SQLException("Registro " + numero + " inexistente!");
			}
			return true;
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	/**
	 * Retorna todas as compras.
	 */
	public String[] getAll() throws LojaException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List numeroList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				numeroList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[numeroList.size()];
			return (String[]) numeroList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}

	/**
	 * Retorna compras que combinam com determinado status.
	 */
	public String[] getByStatus(String status) throws LojaException {
		String sql = SQL_STATUS;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List numeroList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, status);
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				numeroList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[numeroList.size()];
			return (String[]) numeroList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
